"""Add a User.email column

Revision ID: 52dcf7261a86
Revises: 479aaacfc5c7
Create Date: 2015-09-02 12:37:10.516974

"""

# revision identifiers, used by Alembic.
revision = '52dcf7261a86'
down_revision = '479aaacfc5c7'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('email', sa.VARCHAR(),
                                     nullable=True))


def downgrade():
    op.drop_column('users', 'email')



"""Non-nullable fields for the Release.

Revision ID: 479aaacfc5c7
Revises: 22f1e11fb509
Create Date: 2015-08-19 19:46:40.785149

"""

# revision identifiers, used by Alembic.
revision = '479aaacfc5c7'
down_revision = '22f1e11fb509'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column("releases", "version", nullable=False)
    op.alter_column("releases", "branch", nullable=False)


def downgrade():
    op.alter_column("releases", "version", nullable=True)
    op.alter_column("releases", "branch", nullable=True)



"""Default Update.pushed to False

Revision ID: 343bb26e7012
Revises: 1eb754722e44
Create Date: 2013-10-15 16:08:46.473476

"""

# revision identifiers, used by Alembic.
revision = '343bb26e7012'
down_revision = '1eb754722e44'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import transaction

from bodhi.server.models import Base, Update


def upgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    with transaction.manager:
        updates = db.query(Update)

        for u in updates:
            if u.pushed is None:
                u.pushed = False


def downgrade():
    # There's really nothing to downgrade here
    pass



"""Release tags are in the DB

Revision ID: 295f950683ed
Revises: 105840e66024
Create Date: 2014-06-02 18:12:07.541496

"""

# revision identifiers, used by Alembic.
revision = '295f950683ed'
down_revision = '105840e66024'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import transaction

from bodhi.server.models import Base, Release

def upgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    # Add the new columns
    op.add_column('releases', sa.Column('stable_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('testing_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('candidate_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('pending_testing_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('pending_stable_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('override_tag', sa.UnicodeText()))

    # Add values for all releases in those new columns
    with transaction.manager:
        for r in db.query(Release):
            r.stable_tag = "%s-updates" % r.dist_tag
            r.testing_tag = "%s-testing" % r.stable_tag
            r.candidate_tag = "%s-candidate" % r.stable_tag
            r.pending_testing_tag = "%s-pending" % r.testing_tag
            r.pending_stable_tag = "%s-pending" % r.stable_tag
            r.override_tag = "%s-override" % r.dist_tag

    # Now make the new columns not-nullable
    op.alter_column('releases', 'stable_tag', nullable=False)
    op.alter_column('releases', 'testing_tag', nullable=False)
    op.alter_column('releases', 'candidate_tag', nullable=False)
    op.alter_column('releases', 'pending_testing_tag', nullable=False)
    op.alter_column('releases', 'pending_stable_tag', nullable=False)
    op.alter_column('releases', 'override_tag', nullable=False)

    # And drop the old columns
    op.drop_column('releases', '_stable_tag')
    op.drop_column('releases', '_testing_tag')
    op.drop_column('releases', '_candidate_tag')

def downgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    # Add the old columns
    op.add_column('releases', sa.Column('_stable_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('_testing_tag', sa.UnicodeText()))
    op.add_column('releases', sa.Column('_candidate_tag', sa.UnicodeText()))

    with transaction.manager:
        for r in db.query(Release):
            r._stable_tag = r.stable_tag
            r._testing_tag = r.testing_tag
            r._candidate_tag = r.candidate_tag

    # And drop the new columns
    op.drop_column('releases', 'stable_tag')
    op.drop_column('releases', 'testing_tag')
    op.drop_column('releases', 'candidate_tag')
    op.drop_column('releases', 'pending_testing_tag')
    op.drop_column('releases', 'pending_stable_tag')
    op.drop_column('releases', 'override_tag')



"""Change Package.name to UnicodeText instead of Unicode(50)

Revision ID: 22f1e11fb509
Revises: 1c58aa468b17
Create Date: 2015-07-06 15:39:13.768827

"""

# revision identifiers, used by Alembic.
revision = '22f1e11fb509'
down_revision = '1c58aa468b17'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.types import UnicodeText, Unicode


def upgrade():
    op.alter_column(table_name='packages', column_name='name', type_=UnicodeText)


def downgrade():
    op.alter_column(table_name='packages', column_name='name', type_=Unicode(50))



"""Add column date_locked to Update table

Revision ID: 3c72757fa59e
Revises: 3aae6532b560
Create Date: 2016-05-15 11:33:14.849331

"""

# revision identifiers, used by Alembic.
revision = '3c72757fa59e'
down_revision = 'a1d8dae7cc'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('updates', sa.Column('date_locked', sa.DateTime(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('updates', 'date_locked')
    ### end Alembic commands ###



#!/usr/bin/env python
import __main__
__requires__ = __main__.__requires__ = 'WebOb>=1.4.1'
import pkg_resources

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
This script pickles all updates/bugs/cves/comments and writes it out to disk
in the format of bodhi-pickledb-YYYYMMDD.HHMM
"""

__requires__ = 'bodhi'

import sys

import cPickle as pickle

from progressbar import ProgressBar, SimpleProgress, Percentage, Bar
from pyramid.paster import setup_logging
setup_logging('/etc/bodhi/production.ini')

from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

from bodhi.server.util import get_critpath_pkgs
import bodhi


def load_sqlalchemy_db():
    print "\nLoading pickled database %s" % sys.argv[2]
    db = file(sys.argv[2], 'r')
    data = pickle.load(db)

    import transaction
    from bodhi.server.models import Base
    from bodhi.server.models import Release, Update, Build, Comment, User, Bug, CVE
    from bodhi.server.models import Package, Group
    from bodhi.server.models import UpdateType, UpdateStatus, UpdateRequest
    from sqlalchemy import create_engine
    from sqlalchemy.orm.exc import NoResultFound

    # Caches for quick lookup
    releases = {}
    packages = {}
    users = {}
    critpath = {}

    aliases = []

    engine = bodhi.server.config['sqlalchemy.url']
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()

    # Allow filtering of releases to load
    whitelist = []
    if '--release' in sys.argv:
        for r in sys.argv[sys.argv.index('--release') + 1].split(','):
            whitelist.append(r)
        print('whitelist = %r' % whitelist)

    # Legacy format was just a list of update dictionaries
    # Now we'll pull things out into an organized dictionary:
    # {'updates': [], 'releases': []}
    if isinstance(data, dict):
        for release in data['releases']:
            try:
                db.query(Release).filter_by(name=release['name']).one()
            except NoResultFound:
                del(release['metrics'])
                del(release['locked'])
                r = Release(**release)
                r.stable_tag = "%s-updates" % r.dist_tag
                r.testing_tag = "%s-testing" % r.stable_tag
                r.candidate_tag = "%s-candidate" % r.stable_tag
                r.pending_testing_tag = "%s-pending" % r.testing_tag
                r.pending_stable_tag = "%s-pending" % r.stable_tag
                r.override_tag = "%s-override" % r.dist_tag
                db.add(r)
        data = data['updates']

    progress = ProgressBar(widgets=[SimpleProgress(), Percentage(), Bar()])

    for u in progress(data):
        try:
            release = releases[u['release'][0]]
        except KeyError:
            try:
                release = db.query(Release).filter_by(name=u['release'][0]).one()
            except NoResultFound:
                release = Release(name=u['release'][0], long_name=u['release'][1],
                                  id_prefix=u['release'][2],
                                  dist_tag=u['release'][3])
                db.add(release)
            releases[u['release'][0]] = release
            if whitelist:
                if release.name in whitelist:
                    critpath[release.name] = get_critpath_pkgs(release.name.lower())
                    print('%s critpath packages for %s' % (len(critpath[release.name]),
                                                           release.name))
            else:
                critpath[release.name] = get_critpath_pkgs(release.name.lower())
                print('%s critpath packages for %s' % (len(critpath[release.name]),
                                                       release.name))

        if whitelist and release.name not in whitelist:
            continue

        ## Backwards compatbility
        request = u['request']
        if u['request'] == 'move':
            u['request'] = 'stable'
        elif u['request'] == 'push':
            u['request'] = 'testing'
        elif u['request'] == 'unpush':
            u['request'] = 'obsolete'
        if u['approved'] not in (True, False):
            u['approved'] = None
        if u.has_key('update_id'):
            u['updateid'] = u['update_id']
            u['alias'] = u['update_id']

            if u['alias']:
                split = u['alias'].split('-')
                year, id = split[-2:]
                aliases.append((int(year), int(id)))

        if not u.has_key('date_modified'):
            u['date_modified'] = None

        # Port to new enum types
        if u['request']:
            if u['request'] == 'stable':
                u['request'] = UpdateRequest.stable
            elif u['request'] == 'testing':
                u['request'] = UpdateRequest.testing
            else:
                raise Exception("Unknown request: %s" % u['request'])

        if u['type'] == 'bugfix':
            u['type'] = UpdateType.bugfix
        elif u['type'] == 'newpackage':
            u['type'] = UpdateType.newpackage
        elif u['type'] == 'enhancement':
            u['type'] = UpdateType.enhancement
        elif u['type'] == 'security':
            u['type'] = UpdateType.security
        else:
            raise Exception("Unknown type: %r" % u['type'])

        if u['status'] == 'pending':
            u['status'] = UpdateStatus.pending
        elif u['status'] == 'testing':
            u['status'] = UpdateStatus.testing
        elif u['status'] == 'obsolete':
            u['status'] = UpdateStatus.obsolete
        elif u['status'] == 'stable':
            u['status'] = UpdateStatus.stable
        elif u['status'] == 'unpushed':
            u['status'] = UpdateStatus.unpushed
        else:
            raise Exception("Unknown status: %r" % u['status'])

        try:
            update = db.query(Update).filter_by(title=u['title']).one()
            continue
        except NoResultFound:
            update = Update(title=u['title'],
                            date_submitted=u['date_submitted'],
                            date_pushed=u['date_pushed'],
                            date_modified=u['date_modified'],
                            release=release,
                            old_updateid=u['updateid'],
                            alias=u['updateid'],
                            pushed=u['pushed'],
                            notes=u['notes'],
                            karma=u['karma'],
                            type=u['type'],
                            status=u['status'],
                            request=u['request'],
                            )
                            #approved=u['approved'])
            db.add(update)
            db.flush()

            try:
                user = users[u['submitter']]
            except KeyError:
                try:
                    user = db.query(User).filter_by(name=u['submitter']).one()
                except NoResultFound:
                    user = User(name=u['submitter'])
                    db.add(user)
                    db.flush()
                users[u['submitter']] = user
            user.updates.append(update)

        ## Create Package and Build objects
        for pkg, nvr in u['builds']:
            try:
                package = packages[pkg]
            except KeyError:
                try:
                    package = db.query(Package).filter_by(name=pkg).one()
                except NoResultFound:
                    package = Package(name=pkg)
                    db.add(package)
                packages[pkg] = package
            if package.name in critpath[update.release.name]:
                update.critpath = True
            try:
                build = db.query(Build).filter_by(nvr=nvr).one()
            except NoResultFound:
                build = Build(nvr=nvr, package=package)
                db.add(build)
                update.builds.append(build)

        ## Create all Bugzilla objects for this update
        for bug_num, bug_title, security, parent in u['bugs']:
            try:
                bug = db.query(Bug).filter_by(bug_id=bug_num).one()
            except NoResultFound:
                bug = Bug(bug_id=bug_num, security=security, parent=parent,
                          title=bug_title)
                db.add(bug)
            update.bugs.append(bug)

        ## Create all CVE objects for this update
        for cve_id in u['cves']:
            try:
                cve = db.query(CVE).filter_by(cve_id=cve_id).one()
            except NoResultFound:
                cve = CVE(cve_id=cve_id)
                db.add(cve)
            update.cves.append(cve)

        ## Create all Comments for this update
        for c in u['comments']:
            try:
                timestamp, author, text, karma, anonymous = c
            except ValueError:
                timestamp, author, text, karma = c
                anonymous = '@' in author

            comment = Comment(timestamp=timestamp, text=text,
                              karma=karma, anonymous=anonymous)
            db.add(comment)
            db.flush()
            update.comments.append(comment)
            if anonymous:
                name = u'anonymous'
            else:
                name = author
            group = None
            if not anonymous and ' (' in name:
                split = name.split(' (')
                name = split[0]
                group = split[1][:-1]
                assert group, name
            try:
                user = users[name]
            except KeyError:
                try:
                    user = db.query(User).filter_by(name=name).one()
                except NoResultFound:
                    user = User(name=name)
                    db.add(user)
                    db.flush()
                users[name] = user

            comment.user = user

            if group:
                try:
                    group = db.query(Group).filter_by(name=group).one()
                except NoResultFound:
                    group = Group(name=group)
                    db.add(group)
                    db.flush()
                user.groups.append(group)

        db.flush()

    # Hack to get the Bodhi2 alias generator working with bodhi1 data.
    # The new generator assumes that the alias is assigned at submission time, as opposed to push time.
    year, id = max(aliases)
    print('Highest alias = %r %r' % (year, id))
    up = db.query(Update).filter_by(alias=u'FEDORA-%s-%s' % (year, id)).one()
    print(up.title)
    up.date_submitted = up.date_pushed
    db.flush()

    transaction.commit()

    print("\nDatabase migration complete!")
    print(" * %d updates" % db.query(Update).count())
    print(" * %d builds" % db.query(Build).count())
    print(" * %d comments" % db.query(Comment).count())
    print(" * %d users" % db.query(User).count())
    print(" * %d bugs" % db.query(Bug).count())
    print(" * %d CVEs" % db.query(CVE).count())


def usage():
    print "Usage: ./pickledb.py [ load <file> ] [--release <r1,r2>]"
    sys.exit(-1)


def main():
    if len(sys.argv) < 2:
        usage()
    elif sys.argv[1] == 'load' and len(sys.argv) == 3:
        load_sqlalchemy_db()
    elif sys.argv[1] == 'migrate' and len(sys.argv) >= 3:
        load_sqlalchemy_db()
    else:
        usage()

if __name__ == '__main__':
    main()



"""Set testing/stable dates for all old updates.

Revision ID: 3dbed75df3fe
Revises: 13cfca635b99
Create Date: 2015-09-03 14:06:11.762119

"""

# revision identifiers, used by Alembic.
revision = '3dbed75df3fe'
down_revision = '13cfca635b99'

import transaction

from alembic import op
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

from bodhi.server.models import Update, Base

import logging
log = logging.getLogger('alembic.migration')

testing = u'This update has been pushed to testing'
stable = u'This update has been pushed to stable'

def upgrade():
    log.warn("Skipping.  Do this by hand by uncommenting and running in tmux.")
    #log.info("Getting session for data upgrade.")
    #engine = op.get_bind().engine
    #Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    #Session.configure(bind=engine)
    #db = Session()
    #Base.metadata.bind = engine

    #with transaction.manager:
    #    log.info("Querying for all updates ever.")
    #    total = db.query(Update).count()
    #    log.info(" %i" % total)
    #    log.info("OK")
    #    for i, update in enumerate(db.query(Update).yield_per(1000).enable_eagerloads(False)):
    #        if i % 100 == 0:
    #            log.info(" Considering update (%i/%i) %r" % (
    #                i, total, update.title))
    #        for comment in update.comments:
    #            if comment.user.name == u'bodhi':
    #                if comment.text.startswith(testing):
    #                    update.date_testing = comment.timestamp
    #                elif comment.text.startswith(stable):
    #                    update.date_stable = comment.timestamp
    #    log.info("Done.  Committing..")


def downgrade():
    # NOPE!
    pass



"""Add karma_critpath column.

Revision ID: 105840e66024
Revises: 5834b7b7e4cb
Create Date: 2014-05-08 10:00:21.609055

"""

# revision identifiers, used by Alembic.
revision = '105840e66024'
down_revision = '5834b7b7e4cb'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('comments', sa.Column('karma_critpath', sa.Integer(), nullable=True))


def downgrade():
    op.drop_column('comments', 'karma_critpath')



"""Add a unique constraint to the Update.title

Revision ID: 3aae6532b560
Revises: 83c7716b12cf
Create Date: 2016-03-01 19:03:30.088491

"""

# revision identifiers, used by Alembic.
revision = '3aae6532b560'
down_revision = '83c7716b12cf'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker, aliased
from zope.sqlalchemy import ZopeTransactionExtension

import transaction
import logging
log = logging.getLogger('alembic.migration')

from bodhi.server.models import Update, Base


def upgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    # Remove updates with duplicate titles and empty builds
    # This bug has been resolved with https://github.com/fedora-infra/bodhi/pull/809
    with transaction.manager:
        update_alias = aliased(Update)
        dupes = db.query(Update, update_alias).\
                join((update_alias, Update.id > update_alias.id)).\
                filter(Update.title==update_alias.title).all()
        for pair in dupes:
            for up in pair:
                if len(up.builds) == 0:
                    log.info('Deleting %s (%s)' % (up.title, up.alias))
                    for c in up.comments:
                        db.delete(c)
                    db.delete(up)

    op.drop_index('ix_updates_title', table_name='updates')
    op.create_index(op.f('ix_updates_title'), 'updates', ['title'], unique=True)


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_updates_title'), table_name='updates')
    op.create_index('ix_updates_title', 'updates', ['title'], unique=False)
    ### end Alembic commands ###



"""Drop unused columns

Revision ID: 7f3bd827e70
Revises: 18cad09c8ab6
Create Date: 2014-05-06 15:40:32.925576

"""

# revision identifiers, used by Alembic.
revision = '7f3bd827e70'
down_revision = '18cad09c8ab6'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_column(u'updates', 'qa_approval_date')
    op.drop_column(u'updates', 'security_approval_date')
    op.drop_column(u'updates', 'qa_approved')
    op.drop_column(u'updates', 'security_approved')
    op.drop_column(u'updates', 'releng_approval_date')
    op.drop_column(u'updates', 'releng_approved')


def downgrade():
    op.add_column(u'updates', sa.Column('releng_approved', sa.BOOLEAN(), nullable=True))
    op.add_column(u'updates', sa.Column('releng_approval_date', sa.DATETIME(), nullable=True))
    op.add_column(u'updates', sa.Column('security_approved', sa.BOOLEAN(), nullable=True))
    op.add_column(u'updates', sa.Column('qa_approved', sa.BOOLEAN(), nullable=True))
    op.add_column(u'updates', sa.Column('security_approval_date', sa.DATETIME(), nullable=True))
    op.add_column(u'updates', sa.Column('qa_approval_date', sa.DATETIME(), nullable=True))



#!/usr/bin/python
"""
Print out a list of builds that need signing.
"""

__requires__ = 'bodhi'

import pkg_resources
import os
import sys
import pickle

import optparse

from fedora.client import BodhiClient

bodhi = BodhiClient(base_url='http://localhost/updates/')


parser = optparse.OptionParser()
parser.add_option('-e', '--epel',    action='store_true', help='Output EPEL builds')
parser.add_option('-f', '--fedora',  action='store_true', help='Output Fedora builds')
parser.add_option('-v', '--verbose', action='store_true', help='Verbose output')
opts, args = parser.parse_args()


def signable_builds(release):
    locked = set()

    # Load the bodhi masher state
    lock = '/mnt/koji/mash/updates/MASHING-%s' % release.id_prefix
    if os.path.exists(lock):
        state = pickle.load(file(lock))
        for update in state['updates']:
            for build in update.split(','):
                locked.add(build)

    for req in ('stable', 'testing'):
        updates = bodhi.query(request=req, release=release['name'])
        for update in updates['updates']:
            for build in update['builds']:
                if build['nvr'] not in locked:
                    yield build['nvr']

if __name__ == '__main__':
    for release in bodhi.get_releases()['releases']:

        if opts.verbose:
            print("release: %s" % release)

        if opts.epel and release['id_prefix'] == 'FEDORA-EPEL':
            pass
        elif opts.fedora and release['id_prefix'] == 'FEDORA':
            pass
        else:
            continue

        for build in signable_builds(release):
            print(build)

# vim: ts=4 sw=4 ai expandtab


"""Introduce release states

Revision ID: 3b85a0db1a5d
Revises: 2519ca5cbba4
Create Date: 2014-06-02 19:50:59.299902

"""

# revision identifiers, used by Alembic.
revision = '3b85a0db1a5d'
down_revision = '2519ca5cbba4'

from alembic import op
import sqlalchemy as sa

from bodhi.server.models.models import ReleaseState


def upgrade():
    ReleaseState.db_type().create(bind=op.get_bind())
    op.add_column('releases', sa.Column('state', ReleaseState.db_type(), nullable=False, server_default='disabled'))
    op.drop_column('releases', 'locked')


def downgrade():
    op.drop_column('releases', 'state')
    op.add_column('releases', sa.Column('locked', sa.BOOLEAN(), default=False))
    ReleaseState.db_type().drop(bind=op.get_bind())



"""Add a Build.epoch column

Revision ID: 83c7716b12cf
Revises: 6383ec38980
Create Date: 2016-03-01 19:05:54.649231

"""

# revision identifiers, used by Alembic.
revision = '83c7716b12cf'
down_revision = '6383ec38980'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('builds', sa.Column('epoch', sa.Integer, default=0))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('builds', 'epoch')
    ### end Alembic commands ###



"""Default Update.severity to unspecified

Revision ID: 387fda7a1ff0
Revises: 343bb26e7012
Create Date: 2013-10-15 17:06:03.433125

"""

# revision identifiers, used by Alembic.
revision = '387fda7a1ff0'
down_revision = '343bb26e7012'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import transaction

from bodhi.server.models import Base, Update, UpdateSeverity




def upgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    with transaction.manager:
        updates = db.query(Update)

        for u in updates:
            if u.severity is None:
                u.severity = UpdateSeverity.unspecified


def downgrade():
    # There's really nothing to downgrade here
    pass



"""Add resultsdb requirements

Revision ID: 46f84789bec4
Revises: 4efd62c610a6
Create Date: 2014-10-29 11:48:29.622675

"""

# revision identifiers, used by Alembic.
revision = '46f84789bec4'
down_revision = '4efd62c610a6'

from alembic import op
import sqlalchemy as sa
import bodhi.server.models as m
import transaction

from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import logging
log = logging.getLogger('alembic.migration')



def upgrade():
    op.add_column('packages', sa.Column('requirements', sa.UnicodeText(), nullable=True))
    op.add_column('stacks', sa.Column('requirements', sa.UnicodeText(), nullable=True))
    op.add_column('updates', sa.Column('requirements', sa.UnicodeText(), nullable=True))

    # And then, for each one, apply the defaults.
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    m.Base.metadata.bind = engine

    # We can set these requirements to whatever seems best.  I have them as the
    # only two taskotron tests I know of right now for testing, but before
    # launch we could change this to "just" depcheck, or something else.  It is
    # a policy question for FESCo, I think.  -- threebean
    default_reqs = 'depcheck upgradepath'
    ## Some day we'll have rpmgrill, and that will be cool.  Ask tflink.
    #default_reqs = 'depcheck upgradepath rpmlint'

    with transaction.manager:
        log.info("Applying default reqs %r to all stacks." % default_reqs)
        for stack in db.query(m.Stack).all():
            stack.requirements = default_reqs

        log.info("Applying default reqs %r to all packages." % default_reqs)
        for package in db.query(m.Package).all():
            package.requirements = default_reqs

        # We don't set requirements retroactively for updates though.  That
        # would be dishonest if they had already been pushed.


def downgrade():
    op.drop_column('updates', 'requirements')
    op.drop_column('stacks', 'requirements')
    op.drop_column('packages', 'requirements')



"""Add bug and testcase requirements.

Revision ID: 1c58aa468b17
Revises: 46f84789bec4
Create Date: 2014-10-29 14:53:13.033699

"""

# revision identifiers, used by Alembic.
revision = '1c58aa468b17'
down_revision = '46f84789bec4'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('updates',
                  sa.Column('require_bugs', sa.Boolean(), nullable=True))
    op.add_column('updates',
                  sa.Column('require_testcases', sa.Boolean(), nullable=True))


def downgrade():
    op.drop_column('updates', 'require_testcases')
    op.drop_column('updates', 'require_bugs')



"""Add stack_user_table and stack_group_table

Revision ID: 4efd62c610a6
Revises: d58f9439269
Create Date: 2014-10-06 13:10:55.002150

"""

# revision identifiers, used by Alembic.
revision = '4efd62c610a6'
down_revision = 'd58f9439269'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('stack_group_table',
        sa.Column('stack_id', sa.Integer(), nullable=True),
        sa.Column('group_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['group_id'], ['groups.id'], ),
        sa.ForeignKeyConstraint(['stack_id'], ['stacks.id'], )
    )
    op.create_table('stack_user_table',
        sa.Column('stack_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['stack_id'], ['stacks.id'], ),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], )
    )


def downgrade():
    op.drop_table('stack_user_table')
    op.drop_table('stack_group_table')



"""Add a show_popups attribute for users.

Revision ID: 6383ec38980
Revises: 2defe1107259
Create Date: 2015-10-27 13:33:34.193337

"""

# revision identifiers, used by Alembic.
revision = '6383ec38980'
down_revision = '2defe1107259'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import text


def upgrade():
    op.add_column('users', sa.Column(
        'show_popups', sa.Boolean(), server_default=text('TRUE')))


def downgrade():
    op.drop_column('users', 'show_popups')



"""Index the Update.title and Update.date_submitted columns.

Revision ID: 70a58ae9f90
Revises: 3dbed75df3fe
Create Date: 2015-09-04 21:26:58.796325

"""

# revision identifiers, used by Alembic.
revision = '70a58ae9f90'
down_revision = '3dbed75df3fe'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_index(op.f('ix_updates_title'), 'updates', ['title'], unique=False)
    op.create_index(op.f('ix_updates_date_submitted'), 'updates', ['date_submitted'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_updates_title'), table_name='updates')
    op.drop_index(op.f('ix_updates_date_submitted'), table_name='updates')



"""Remove the Release.metrics PickleCol

Revision ID: 5735e34a4ba
Revises: 6a438fe37fd
Create Date: 2014-06-16 10:06:40.082113

"""

# revision identifiers, used by Alembic.
revision = '5735e34a4ba'
down_revision = '6a438fe37fd'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('releases', 'metrics')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('releases', sa.Column('metrics', postgresql.BYTEA(), autoincrement=False, nullable=True))
    ### end Alembic commands ###



"""Add the critpath column to the update model

Revision ID: 4d7508f9cabc
Revises: None
Create Date: 2013-08-12 17:59:28.477469

"""

# revision identifiers, used by Alembic.
revision = '4d7508f9cabc'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('updates', sa.Column('critpath', sa.Boolean(), nullable=True))


def downgrade():
    op.drop_column('updates', 'critpath')



"""Add a Release.branch column

Revision ID: 2519ca5cbba4
Revises: 295f950683ed
Create Date: 2014-06-03 13:15:06.129856

"""

# revision identifiers, used by Alembic.
revision = '2519ca5cbba4'
down_revision = '295f950683ed'

from alembic import op
import sqlalchemy as sa
import transaction
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

from bodhi.server.models import Release, Base


def upgrade():
    op.add_column('releases', sa.Column('branch', sa.Unicode(length=10)))
    op.create_unique_constraint(None, 'releases', ['branch'])

    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    with transaction.manager:
        for release in db.query(Release).all():
            release.branch = release.name.lower()


def downgrade():
    op.drop_constraint(None, 'releases')
    op.drop_column('releases', 'branch')



"""Add the stacks table.

Revision ID: 2861bb4e77b1
Revises: 5735e34a4ba
Create Date: 2014-08-08 11:45:25.303638

"""

# revision identifiers, used by Alembic.
revision = '2861bb4e77b1'
down_revision = '5735e34a4ba'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('stacks',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.UnicodeText(), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name')
    )
    op.add_column(u'packages', sa.Column('stack_id', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column(u'packages', 'stack_id')
    op.drop_table('stacks')
    ### end Alembic commands ###



"""Default Update.critpath to False

Revision ID: 1eb754722e44
Revises: 4d7508f9cabc
Create Date: 2013-08-27 11:15:41.938750

"""

# revision identifiers, used by Alembic.
revision = '1eb754722e44'
down_revision = '4d7508f9cabc'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import transaction

from bodhi.server.models import Base, Release, Update
from bodhi.server.util import get_critpath_pkgs


def upgrade():
    engine = op.get_bind()
    Base.metadata.bind = engine
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()

    critpath_pkgs = {}
    for release in db.query(Release):
        relname = release.name
        critpath_pkgs[relname] = sorted(get_critpath_pkgs(relname.lower()))

    with transaction.manager:
        updates = db.query(Update)

        for up in updates:
            for build in up.builds:
                if build.package.name in critpath_pkgs[up.release.name]:
                    up.critpath = True
                    break

            else:
                up.critpath = False


def downgrade():
    engine = op.get_bind()
    Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    Session.configure(bind=engine)
    db = Session()
    Base.metadata.bind = engine

    with transaction.manager:
        updates = db.query(Update)

        for u in updates:
            u.critpath = None



"""Set the pushed flag.

Revision ID: 2defe1107259
Revises: 70a58ae9f90
Create Date: 2015-09-11 15:39:03.656516

"""

# revision identifiers, used by Alembic.
revision = '2defe1107259'
down_revision = '70a58ae9f90'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension

import transaction

import logging
log = logging.getLogger('alembic.migration')

from bodhi.server.models import Base, Update

def upgrade():
    log.warn("Skipping.  Do this by hand by uncommenting and running in tmux.")
    #log.info("Getting session for data upgrade.")
    #engine = op.get_bind().engine
    #Session = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    #Session.configure(bind=engine)
    #db = Session()
    #Base.metadata.bind = engine

    #with transaction.manager:
    #    log.info("Querying for all updates with pushed!=True.")
    #    query = db.query(Update).filter(Update.pushed!=True)
    #    total = query.count()
    #    log.info(" %i" % total)
    #    log.info("OK")
    #    for i, update in enumerate(query.yield_per(1000).enable_eagerloads(False)):
    #        if i % 100 == 0:
    #            log.info(" Considering update (%i/%i) %r" % (
    #                i, total, update.title))
    #        if update.date_stable or update.date_testing:
    #            update.pushed = True
    #    log.info("Done.  Committing..")


def downgrade():
    # NOPE!
    pass



import __main__
__requires__ = __main__.__requires__ = 'WebOb>=1.4.1'
import pkg_resources

# The following two imports are required to shut up an
# atexit error when running tests with python 2.7
import logging
import multiprocessing

import os
import sys

from setuptools import setup, find_packages
import setuptools.command.egg_info


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
VERSION = '2.3.1'
# Possible options are at https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 2.7',
    'Topic :: System :: Software Distribution']
LICENSE = 'GPLv2+'
MAINTAINER = 'Fedora Infrastructure Team'
MAINTAINER_EMAIL = 'infrastructure@lists.fedoraproject.org'
PLATFORMS = ['Fedora', 'GNU/Linux']
URL = 'https://github.com/fedora-infra/bodhi'

server_requires = [
    'pyramid',
    'pyramid_mako',
    'pyramid_debugtoolbar',
    'pyramid_tm',
    'waitress',
    'colander',
    'cornice',

    'python-openid',
    'pyramid_fas_openid',
    'packagedb-cli',

    'sqlalchemy',
    'zope.sqlalchemy',

    'webhelpers',
    'progressbar',

    'bunch',

    # for captchas
    'cryptography',
    'Pillow',

    # Useful tools
    'kitchen',
    'python-fedora',
    'pylibravatar',
    'pyDNS',
    'dogpile.cache',
    'arrow',
    'markdown',

    # i18n, that we're not actually doing yet.
    #'Babel',
    #'lingua',

    # External resources
    'python-bugzilla',
    'simplemediawiki',

    # "python setup.py test" needs one of fedmsg's setup.py extra_requires
    'fedmsg[consumers]',
    # The masher needs fedmsg-atomic-composer
    'fedmsg-atomic-composer >= 2016.3',

    'Sphinx',

    'WebOb>=1.4.1',
    ]

if sys.version_info[:3] < (2,7,0):
    server_requires.append('importlib')

if sys.version_info[:3] < (2,5,0):
    server_requires.append('pysqlite')


setuptools.command.egg_info.manifest_maker.template = 'BODHI_MANIFEST.in'


setup(name='bodhi',
      version=VERSION,
      description='bodhi common package',
      long_description=README,
      classifiers=CLASSIFIERS,
      license=LICENSE,
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      platforms=PLATFORMS,
      url=URL,
      keywords='fedora',
      packages=['bodhi'],
      include_package_data=True,
      zip_safe=False,
      install_requires = [],
      tests_require = [
          'flake8',
          'nose',
          'nose-cov',
          'webtest',
          'mock'
      ],
      test_suite="nose.collector",
      )


setuptools.command.egg_info.manifest_maker.template = 'CLIENT_MANIFEST.in'


setup(name='bodhi-client',
      version=VERSION,
      description='bodhi client',
      long_description=README,
      classifiers=CLASSIFIERS,
      license=LICENSE,
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      platforms=PLATFORMS,
      url=URL,
      keywords='fedora',
      packages=['bodhi.client'],
      include_package_data=False,
      zip_safe=False,
      install_requires = ['click'],
      entry_points = """\
      [console_scripts]
      bodhi = bodhi.client:cli
      """,
      )


setuptools.command.egg_info.manifest_maker.template = 'SERVER_MANIFEST.in'
# Due to https://github.com/pypa/setuptools/issues/808, we need to include the bodhi superpackage
# and then remove it if we want find_packages() to find the bodhi.server package and its
# subpackages without including the bodhi top level package.
server_packages = find_packages(
    exclude=['bodhi.client', 'bodhi.client.*', 'bodhi.tests', 'bodhi.tests.*'])
server_packages.remove('bodhi')


setup(name='bodhi-server',
      version=VERSION,
      description='bodhi server',
      long_description=README,
      classifiers=CLASSIFIERS + [
        "Framework :: Pyramid",
        'Programming Language :: JavaScript',
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application"],
      license=LICENSE,
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      platforms=PLATFORMS,
      url=URL,
      keywords='web fedora pyramid',
      packages=server_packages,
      include_package_data=True,
#      script_args=sys.argv.extend(['--template', 'TEST']),
      zip_safe=False,
      install_requires = server_requires,
      message_extractors = { '.': [
          #('**.py', 'lingua_python', None),
          #('**.mak', 'lingua_xml', None),
      ]},
      entry_points = """\
      [paste.app_factory]
      main = bodhi.server:main
      [console_scripts]
      initialize_bodhi_db = bodhi.server.scripts.initializedb:main
      bodhi-push = bodhi.server.push:push
      bodhi-expire-overrides = bodhi.server.scripts.expire_overrides:main
      bodhi-untag-branched = bodhi.server.scripts.untag_branched:main
      bodhi-approve-testing = bodhi.server.scripts.approve_testing:main
      bodhi-manage-releases = bodhi.server.scripts.manage_releases:main
      [moksha.consumer]
      masher = bodhi.server.consumers.masher:Masher
      updates = bodhi.server.consumers.updates:UpdatesHandler
      signed = bodhi.server.consumers.signed:SignedHandler
      """,
      paster_plugins=['pyramid'],
      )



"""Add buildroot overrides

Revision ID: 6a438fe37fd
Revises: 3b85a0db1a5d
Create Date: 2014-06-04 18:26:39.899842

"""

# revision identifiers, used by Alembic.
revision = '6a438fe37fd'
down_revision = '3b85a0db1a5d'

from datetime import datetime

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('buildroot_overrides',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('build_id', sa.Integer(), nullable=False),
        sa.Column('submitter_id', sa.Integer(), nullable=False),
        sa.Column('notes', sa.Unicode(), nullable=False),
        sa.Column('submission_date', sa.DateTime(), default=datetime.utcnow,
                  nullable=False),
        sa.Column('expiration_date', sa.DateTime(), nullable=False),
        sa.Column('expired_date', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['build_id'], ['builds.id'], ),
        sa.ForeignKeyConstraint(['submitter_id'], ['users.id'], ),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('buildroot_overrides')



"""New karma tables

Revision ID: 5834b7b7e4cb
Revises: 7f3bd827e70
Create Date: 2014-05-06 15:42:08.805202

"""

# revision identifiers, used by Alembic.
revision = '5834b7b7e4cb'
down_revision = '7f3bd827e70'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'comment_testcase_assoc',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('karma', sa.Integer(), nullable=True),
        sa.Column('comment_id', sa.Integer(), nullable=True),
        sa.Column('testcase_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['comment_id'], ['comments.id'], ),
        sa.ForeignKeyConstraint(['testcase_id'], ['testcases.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_table(
        'comment_bug_assoc',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('karma', sa.Integer(), nullable=True),
        sa.Column('comment_id', sa.Integer(), nullable=True),
        sa.Column('bug_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['bug_id'], ['bugs.id'], ),
        sa.ForeignKeyConstraint(['comment_id'], ['comments.id'], ),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('comment_bug_assoc')
    op.drop_table('comment_testcase_assoc')



"""Add a description field to the stacks

Revision ID: d58f9439269
Revises: 2861bb4e77b1
Create Date: 2014-08-21 12:50:05.920845

"""

# revision identifiers, used by Alembic.
revision = 'd58f9439269'
down_revision = '2861bb4e77b1'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('stacks', sa.Column('description', sa.UnicodeText(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('stacks', 'description')
    ### end Alembic commands ###



"""Add update.date_testing/stable timestamps

Revision ID: 13cfca635b99
Revises: 52dcf7261a86
Create Date: 2015-09-02 15:55:40.940168

"""

# revision identifiers, used by Alembic.
revision = '13cfca635b99'
down_revision = '52dcf7261a86'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('updates', sa.Column('date_stable', sa.DateTime(), nullable=True))
    op.add_column('updates', sa.Column('date_testing', sa.DateTime(), nullable=True))


def downgrade():
    op.drop_column('updates', 'date_testing')
    op.drop_column('updates', 'date_stable')



"""Add autokarma column

Revision ID: a1d8dae7cc
Revises: 3aae6532b560
Create Date: 2016-05-20 21:30:25.805859

"""

# revision identifiers, used by Alembic.
revision = 'a1d8dae7cc'
down_revision = '3aae6532b560'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('updates', sa.Column('autokarma', sa.Boolean(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('updates', 'autokarma')
    ### end Alembic commands ###
